/***************************************************************************
                          main.c  -  description
                             -------------------
    begin                : Fre Okt 13 16:03:08 CEST 2000
    copyright            : (C) 2000 by Markus Fruehwirth
    email                : 7ox@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "SJBfkt.h"


/* name: calc
 * var: steps for fft, length in s
 * exitcode:0 (for normal termination - returncode)
 * explanation: interpolates given data
*/
int
calc (unsigned long int handover, unsigned long int alltime)
{
  int back;			//returnvalue
  FILE *read;
  FILE *write;
  char *line, *saveline;	//read this line
  unsigned long int *z, *w;	//pointer to time and value
  unsigned long int zz, zw;	//time and value
  int lines;			//eof
  unsigned long int *time, *value;	//time- and valuearray
  int ende;			//break
  int ready;			//for reading
  float dy, *xa, *ya, y, x, all, hand, *xc, *yc;	//interpolation
  int i, j, k;			//counter
  int upper, below, mid;	//borders
  int maxlines;			//all lines

  maxlines = MAXLINE;
  upper = 0;
  below = 0;
  mid = 0;
  back = -1;
  line = malloc (MAXLENLINE * sizeof (char));
  saveline = malloc (MAXLENLINE * sizeof (char));
  lines = 1;
  time = malloc (MAXLINE * sizeof (unsigned long int));
  value = malloc (MAXLINE * sizeof (unsigned long int));
  zw = 0;
  zz = 0;
  z = &zz;
  w = &zw;
  ende = 0;
  ready = 0;
  all = (float) alltime / MICROSEC;	//length in sec
  hand = all / (float) handover;	//steps in sec


// 1. stdin komplett lesen
  read = stdin;


  while (ready == 0)
    {
      saveline = fgets (line, MAXLENLINE, read);
//read a line

      if (saveline == NULL)	//EOF
	ready = 1;

      if (ready == 0)
	{
	  back = sscanf (line, "%lu %lu", z, w);
//time in zz, value in zw
//in array is too small realloc array
	  if (lines == maxlines)
	    {
	      time = realloc (time, (lines + 2) * sizeof (unsigned long int));
	      value =
		realloc (value, (lines + 2) * sizeof (unsigned long int));
	      maxlines = maxlines + 2;
	    }

	  //save in Arrays
	  time[lines] = zz;
	  value[lines] = zw;

	  lines++;
	}
    }
  lines--;
//lines has counted all read lines

  (void) fclose (read);
  free (line);
  free (saveline);

//stdin OK?
  if (zz < alltime)
    ende = 1;			//wrong length of input

//data now in time and value
  if (ende == 0)
    {
      write = stdout;
//write legth in sec and steps
      (void) fprintf (write, "%lu %lu\n", (alltime / MICROSEC), handover);

//copy int array to float unit offset array
      xa = vector (1, lines);
      ya = vector (1, lines);
      xc = vector (1, POLDEG);
      yc = vector (1, POLDEG);
      for (i = 1; i <= (lines); i++)
	{
	  xa[i] = (float) ((float) time[i] / (float) MICROSEC);
	  ya[i] = (float) ((float) value[i] / (float) ADJUST);
	}

      x = hand;
      below = 1;
      upper = 11;
      mid = 1;
      while (x <= all)
	{
	  while (xa[mid] < x)
	    {
	      mid++;
	      if (mid == lines)
		break;
	    }
//mid contains now the index which is closest to the approximation

	  upper = mid + POLDHALF;
	  below = mid - POLDHALF;

	  if (below < 1)
	    {
	      below = 1;
	      upper = POLDEG;
	    }

	  if (upper > lines)
	    {
	      below = lines - POLDEG;
	      upper = lines;
	    }

	  k = 1;
	  for (j = below; j <= (POLDEG + below); j++)
	    {
	      xc[k] = xa[j];
	      yc[k] = ya[j];
	      k++;
	    }
//call polint with arguments
	  polint (xc, yc, POLDEG, x, &y, &dy);
//write output
	  (void) fprintf (write, "%f\n", y);
	  x = x + hand;
	}			//array ready

      (void) fflush (write);
      free_vector (xa, 1, lines);
      free_vector (ya, 1, lines);
      free_vector (xc, 1, POLDEG);
      free_vector (yc, 1, POLDEG);
    }				//ende
  free (time);
  free (value);
  return back;
}


/* name: main
 * var:  steps for fft, length in s
 * exitcode:0 (for normal termination - returncode)
 * explanation: processes file
*/
int
main (int argc, char *argv[])
{
  int back = -1;
  unsigned long int handover, *hand, handover2, *hand2;

  handover = 256;
  hand = &handover;
  handover2 = 5;
  hand2 = &handover2;

//convert arguments
  if (argc == 3)
    {
      back = sscanf (argv[2], "%lu", hand);	//time
      back = sscanf (argv[1], "%lu", hand2);	//length

      if ((handover % 2) != 0)
	{
	  
	    (void) fprintf (stderr,
			    "%s: second argument must be a power of 2\n",
			    argv[0]);
	  exit (EXIT_FAILURE);
	}
      back = calc (handover, handover2 * MICROSEC);
    }
  else
    {
      (void) fprintf (stderr, "%s: wrong Usage: %s time\n", argv[0], argv[0]);
      exit (EXIT_FAILURE);
    }


  return back;
}
